@@ -10,7 +10,7 @@ use constant KEY_WOW64_32 => 0x200;
 use Encode;
 use English qw(-no_match_vars);
 
-use Win32::OLE qw(in CP_UTF8);
+use Win32::OLE qw(in);
 use Win32::OLE::Const;
 use Win32::TieRegistry (
     Delimiter   => '/',
@@ -18,7 +18,11 @@ use Win32::TieRegistry (
     qw/KEY_READ/
 );
 
-Win32::OLE->Option(CP => 'CP_UTF8');
+use File::Temp ();
+use File::Temp qw(:seekable);
+use Win32::Job;
+
+Win32::OLE->Option(CP => Win32::OLE::CP_UTF8);
 
 use FusionInventory::Agent::Tools;
 
@@ -33,12 +37,12 @@ our @EXPORT = qw(
     getRegistryKey
     getWmiObjects
     getLocalCodepage
+    runCommand
 );
 
 sub is64bit {
-
     return
-        any { $_->{AddressWidth} eq 64 } 
+        any { $_->{AddressWidth} eq 64 }
         getWmiObjects(
             class => 'Win32_Processor', properties => [ qw/AddressWidth/ ]
         );
@@ -46,19 +50,14 @@ sub is64bit {
 
 sub getLocalCodepage {
     if (!$localCodepage) {
-        my $lmachine = $Registry->Open('LMachine', {
-            Access => KEY_READ
-        }) or die "Can't open HKEY_LOCAL_MACHINE key: $EXTENDED_OS_ERROR";
-
-        my $codepage =
-            $lmachine->{"SYSTEM\\CurrentControlSet\\Control\\Nls\\CodePage"}
-            or warn;
-
-            $localCodepage = "cp".$codepage->{ACP};
+        $localCodepage =
+            "cp" .
+            getRegistryValue(
+                path => 'HKEY_LOCAL_MACHINE/SYSTEM/CurrentControlSet/Control/Nls/CodePage/ACP'
+            );
     }
 
     return $localCodepage;
-
 }
 
 sub encodeFromRegistry {
@@ -97,14 +96,14 @@ sub getRegistryValue {
     my (%params) = @_;
 
     my ($root, $keyName, $valueName);
-    if ($params{path} =~ /^(HKEY_[^\/\\]+)[\/\\](.+)[\/\\]([^\/]+)/ ) {
+    if ($params{path} =~ m{^(HKEY_\S+)/(.+)/([^/]+)} ) {
         $root      = $1;
         $keyName   = $2;
         $valueName = $3;
     } else {
         $params{logger}->error(
-	    "Failed to parse '$params{path}'. Does it start with HKEY_?"
-	) if $params{logger};
+            "Failed to parse '$params{path}'. Does it start with HKEY_?"
+        ) if $params{logger};
         return;
     }
 
@@ -120,13 +119,13 @@ sub getRegistryKey {
     my (%params) = @_;
 
     my ($root, $keyName);
-    if ($params{path} =~ /^(HKEY_\S+)\/(.+)/ ) {
+    if ($params{path} =~ m{^(HKEY_\S+)/(.+)} ) {
         $root      = $1;
         $keyName   = $2;
     } else {
         $params{logger}->error(
-	    "Failed to parse '$params{path}'. Does it start with HKEY_?"
-	) if $params{logger};
+            "Failed to parse '$params{path}'. Does it start with HKEY_?"
+        ) if $params{logger};
         return;
     }
 
@@ -143,7 +142,7 @@ sub _getRegistryKey {
     ## no critic (ProhibitBitwise)
     my $rootKey = is64bit() ?
         $Registry->Open($params{root}, { Access=> KEY_READ | KEY_WOW64_64 } ) :
-	$Registry->Open($params{root}, { Access=> KEY_READ } )                ;
+        $Registry->Open($params{root}, { Access=> KEY_READ } )                ;
 
     if (!$rootKey) {
         $params{logger}->error(
@@ -156,6 +155,44 @@ sub _getRegistryKey {
     return $key;
 }
 
+sub runCommand {
+    my (%params) = (
+        timeout => 3600 * 2,
+        @_
+    );
+
+    my $job = Win32::Job->new();
+
+    my $buff = File::Temp->new();
+    my $void = File::Temp->new();
+
+    my $args = {
+        stdout    => $buff,
+        stderr    => $params{no_stderr} ? $void : $buff,
+        no_window => 1
+    };
+
+    $job->spawn(
+        "$ENV{SYSTEMROOT}/system32/cmd.exe",
+        "cmd.exe /c $params{command}",
+        $args
+    );
+
+    $job->run($params{timeout});
+
+    $buff->seek(0, SEEK_SET);
+
+    my $exitcode;
+
+    my ($status) = $job->status();
+    foreach my $pid (%$status) {
+        $exitcode = $status->{$pid}{exitcode};
+        last;
+    }
+
+    return ($exitcode, $buff);
+}
+
 1;
 __END__
 
@@ -222,3 +259,23 @@ E.g: HKEY_LOCAL_MACHINE/SOFTWARE/Microsoft/Windows NT/CurrentVersion
 =item logger
 
 =back
+
+=head2 runCommand(%params)
+
+Returns a command in a Win32 Process
+
+=over
+
+=item command the command to run
+
+=item timeout a time in second, default is 3600*2
+
+=item no_stderr ignore STDERR output, default is false
+=back
+
+Return an array
+
+=item exitcode the error code, 293 means a timeout occurred
+
+=item fd a file descriptor on the output
+
